@@ -45,6 +45,8 @@ def test_loc():
 
     pytest.raises(KeyError, lambda: d.loc[1000])
     assert_eq(d.loc[1000:], full.loc[1000:])
+    assert_eq(d.loc[1000:2000], full.loc[1000:2000])
+    assert_eq(d.loc[:-1000], full.loc[:-1000])
     assert_eq(d.loc[-2000:-1000], full.loc[-2000:-1000])
 
     assert sorted(d.loc[5].dask) == sorted(d.loc[5].dask)
@@ -335,7 +337,7 @@ def test_loc_on_numpy_datetimes():
         {"x": [1, 2, 3]}, index=list(map(np.datetime64, ["2014", "2015", "2016"]))
     )
     a = dd.from_pandas(df, 2)
-    a.divisions = list(map(np.datetime64, a.divisions))
+    a.divisions = tuple(map(np.datetime64, a.divisions))
 
     assert_eq(a.loc["2014":"2015"], a.loc["2014":"2015"])
 
@@ -345,7 +347,7 @@ def test_loc_on_pandas_datetimes():
         {"x": [1, 2, 3]}, index=list(map(pd.Timestamp, ["2014", "2015", "2016"]))
     )
     a = dd.from_pandas(df, 2)
-    a.divisions = list(map(pd.Timestamp, a.divisions))
+    a.divisions = tuple(map(pd.Timestamp, a.divisions))
 
     assert_eq(a.loc["2014":"2015"], a.loc["2014":"2015"])
 
