@@ -233,12 +233,14 @@ def test_hash_join(how, shuffle_method):
 
     c = hash_join(a, "y", b, "y", how)
 
+    assert not hlg_layer_topological(c.dask, -1).is_materialized()
     result = c.compute()
     expected = pd.merge(A, B, how, "y")
     list_eq(result, expected)
 
     # Different columns and npartitions
     c = hash_join(a, "x", b, "z", "outer", npartitions=3, shuffle=shuffle_method)
+    assert not hlg_layer_topological(c.dask, -1).is_materialized()
     assert c.npartitions == 3
 
     result = c.compute(scheduler="single-threaded")
@@ -634,6 +636,31 @@ def test_merge_asof_with_empty():
     assert_eq(result_dd, result_df, check_index=False)
 
 
+def test_merge_asof_on_left_right():
+    df1 = pd.DataFrame(
+        {
+            "endofweek": [1, 1, 2, 2, 3, 4],
+            "GroupCol": [1234, 8679, 1234, 8679, 1234, 8679],
+        }
+    )
+    df2 = pd.DataFrame(
+        {"timestamp": [0, 0, 1, 3], "GroupVal": [1234, 1234, 8679, 1234]}
+    )
+
+    # pandas
+    result_df = pd.merge_asof(df1, df2, left_on="endofweek", right_on="timestamp")
+
+    # dask
+    result_dd = dd.merge_asof(
+        dd.from_pandas(df1, npartitions=2),
+        dd.from_pandas(df2, npartitions=2),
+        left_on="endofweek",
+        right_on="timestamp",
+    )
+
+    assert_eq(result_df, result_dd, check_index=False)
+
+
 @pytest.mark.parametrize("join", ["inner", "outer"])
 def test_indexed_concat(join):
     A = pd.DataFrame(
@@ -686,20 +713,58 @@ def test_concat(join):
         result = dd.concat([dd1, dd2], join=join, **kwargs)
         assert_eq(result, expected)
 
-    # test outer only, inner has a problem on pandas side
+
+@pytest.mark.parametrize("join", ["inner", "outer"])
+def test_concat_series(join):
+    pdf1 = pd.DataFrame(
+        {"x": [1, 2, 3, 4, 6, 7], "y": list("abcdef")}, index=[1, 2, 3, 4, 6, 7]
+    )
+    ddf1 = dd.from_pandas(pdf1, 2)
+    pdf2 = pd.DataFrame(
+        {"x": [1, 2, 3, 4, 6, 7], "y": list("abcdef")}, index=[8, 9, 10, 11, 12, 13]
+    )
+    ddf2 = dd.from_pandas(pdf2, 2)
+
+    # different columns
+    pdf3 = pd.DataFrame(
+        {"x": [1, 2, 3, 4, 6, 7], "z": list("abcdef")}, index=[8, 9, 10, 11, 12, 13]
+    )
+    ddf3 = dd.from_pandas(pdf3, 2)
+
+    kwargs = {"sort": False}
+
     for (dd1, dd2, pd1, pd2) in [
-        (ddf1, ddf2, pdf1, pdf2),
-        (ddf1, ddf3, pdf1, pdf3),
-        (ddf1.x, ddf2.x, pdf1.x, pdf2.x),
-        (ddf1.x, ddf3.z, pdf1.x, pdf3.z),
         (ddf1.x, ddf2.x, pdf1.x, pdf2.x),
         (ddf1.x, ddf3.z, pdf1.x, pdf3.z),
     ]:
-        expected = pd.concat([pd1, pd2], **kwargs)
-        result = dd.concat([dd1, dd2], **kwargs)
+        expected = pd.concat([pd1, pd2], join=join, **kwargs)
+        result = dd.concat([dd1, dd2], join=join, **kwargs)
         assert_eq(result, expected)
 
 
+def test_concat_with_operation_remains_hlg():
+    pdf1 = pd.DataFrame(
+        {"x": [1, 2, 3, 4, 6, 7], "y": list("abcdef")}, index=[1, 2, 3, 4, 6, 7]
+    )
+    ddf1 = dd.from_pandas(pdf1, 2)
+
+    pdf2 = pd.DataFrame({"y": list("abcdef")}, index=[8, 9, 10, 11, 12, 13])
+    ddf2 = dd.from_pandas(pdf2, 2)
+
+    # Do some operation which will remain blockwise in the dask case.
+    pdf2["x"] = range(len(pdf2["y"]))
+    # See https://stackoverflow.com/a/60852409
+    ddf2["x"] = ddf2.assign(partition_count=1).partition_count.cumsum() - 1
+    kwargs = {"sort": False}
+
+    expected = pd.concat([pdf1, pdf2], **kwargs)
+    result = dd.concat([ddf1, ddf2], **kwargs)
+    # The third layer is the assignment to column `x`, which should remain
+    # blockwise
+    assert not hlg_layer_topological(result.dask, 2).is_materialized()
+    assert_eq(result, expected)
+
+
 @pytest.mark.parametrize(
     "value_1, value_2",
     [
@@ -753,18 +818,17 @@ def test_merge_columns_dtypes(how, on_index):
         b = b.set_index("A")
         on = None
 
-    with pytest.warns(None) as record:
+    with warnings.catch_warnings(record=True) as record:
+        warnings.simplefilter("always")
         result = dd.merge(
             a, b, on=on, how=how, left_index=left_index, right_index=right_index
         )
-
-    warned = any("merge column data type mismatches" in str(r) for r in record)
+        warned = any("merge column data type mismatches" in str(r) for r in record)
 
     # result type depends on merge operation -> convert to pandas
     result = result if isinstance(result, pd.DataFrame) else result.compute()
 
     has_nans = result.isna().values.any()
-
     assert (has_nans and warned) or not has_nans
 
 
@@ -850,6 +914,26 @@ def test_merge(how, shuffle_method):
     #         pd.merge(A, B, left_index=True, right_on='y'))
 
 
+def test_merge_how_raises():
+    left = pd.DataFrame(
+        {
+            "A": ["A0", "A1", "A2", "A3"],
+            "B": ["B0", "B1", "B2", "B3"],
+        }
+    )
+    right = pd.DataFrame(
+        {
+            "A": ["C0", "C1", "C2", "C3"],
+            "B": ["D0", "D1", "D2", "D3"],
+        }
+    )
+
+    with pytest.raises(
+        ValueError, match="dask.dataframe.merge does not support how='cross'"
+    ):
+        dd.merge(left, right, how="cross")
+
+
 @pytest.mark.parametrize("parts", [(3, 3), (3, 1), (1, 3)])
 @pytest.mark.parametrize("how", ["leftsemi", "leftanti"])
 @pytest.mark.parametrize(
@@ -1599,9 +1683,9 @@ def test_concat_unknown_divisions():
     with pytest.raises(ValueError):
         dd.concat([aa, cc], axis=1)
 
-    with pytest.warns(None) as record:
+    with warnings.catch_warnings(record=True) as record:
         dd.concat([aa, bb], axis=1, ignore_unknown_divisions=True)
-        assert len(record) == 0
+    assert not record
 
 
 def test_concat_unknown_divisions_errors():
@@ -1837,12 +1921,10 @@ def test_concat5():
     for case in cases:
         pdcase = [c.compute() for c in case]
 
-        with pytest.warns(None):
-            # some cases will raise warning directly from pandas
-            assert_eq(
-                dd.concat(case, interleave_partitions=True),
-                pd.concat(pdcase, sort=False),
-            )
+        assert_eq(
+            dd.concat(case, interleave_partitions=True),
+            pd.concat(pdcase, sort=False),
+        )
 
         assert_eq(
             dd.concat(case, join="inner", interleave_partitions=True),
@@ -2009,19 +2091,17 @@ def test_concat_datetimeindex():
     assert_eq(result, expected)
 
 
-def check_with_warning(dask_obj, dask_append, pandas_obj, pandas_append):
-    with pytest.warns(None) as record:
+def check_append_with_warning(dask_obj, dask_append, pandas_obj, pandas_append):
+    if PANDAS_GT_140:
+        with pytest.warns(FutureWarning, match="append method is deprecated"):
+            expected = pandas_obj.append(pandas_append)
+            result = dask_obj.append(dask_append)
+            assert_eq(result, expected)
+    else:
         expected = pandas_obj.append(pandas_append)
         result = dask_obj.append(dask_append)
         assert_eq(result, expected)
 
-    if PANDAS_GT_140:
-        for w in record:
-            assert w.category == FutureWarning
-            assert "append method is deprecated" in str(w.message)
-    else:
-        assert not record
-
     return result
 
 
@@ -2040,20 +2120,20 @@ def test_append():
 
     s = pd.Series([7, 8], name=6, index=["a", "b"])
 
-    check_with_warning(ddf, s, df, s)
-    check_with_warning(ddf, ddf2, df, df2)
-    check_with_warning(ddf.a, ddf2.a, df.a, df2.a)
+    check_append_with_warning(ddf, s, df, s)
+    check_append_with_warning(ddf, ddf2, df, df2)
+    check_append_with_warning(ddf.a, ddf2.a, df.a, df2.a)
 
     # different columns
-    check_with_warning(ddf, ddf3, df, df3)
-    check_with_warning(ddf.a, ddf3.b, df.a, df3.b)
+    check_append_with_warning(ddf, ddf3, df, df3)
+    check_append_with_warning(ddf.a, ddf3.b, df.a, df3.b)
 
     # dask + pandas
-    check_with_warning(ddf, df2, df, df2)
-    check_with_warning(ddf.a, df2.a, df.a, df2.a)
+    check_append_with_warning(ddf, df2, df, df2)
+    check_append_with_warning(ddf.a, df2.a, df.a, df2.a)
 
-    check_with_warning(ddf, df3, df, df3)
-    check_with_warning(ddf.a, df3.b, df.a, df3.b)
+    check_append_with_warning(ddf, df3, df, df3)
+    check_append_with_warning(ddf.a, df3.b, df.a, df3.b)
 
 
 def test_append2():
@@ -2082,20 +2162,20 @@ def test_append2():
     ddf3 = dd.DataFrame(dsk, "y", meta, [None, None])
     df3 = ddf3.compute()
 
-    check_with_warning(ddf1, ddf2, df1, df2)
-    check_with_warning(ddf2, ddf1, df2, df1)
+    check_append_with_warning(ddf1, ddf2, df1, df2)
+    check_append_with_warning(ddf2, ddf1, df2, df1)
 
     # different columns
-    check_with_warning(ddf1, ddf3, df1, df3)
-    check_with_warning(ddf3, ddf1, df3, df1)
+    check_append_with_warning(ddf1, ddf3, df1, df3)
+    check_append_with_warning(ddf3, ddf1, df3, df1)
 
     # Dask + pandas
-    check_with_warning(ddf1, df2, df1, df2)
-    check_with_warning(ddf2, df1, df2, df1)
+    check_append_with_warning(ddf1, df2, df1, df2)
+    check_append_with_warning(ddf2, df1, df2, df1)
 
     # different columns
-    check_with_warning(ddf1, df3, df1, df3)
-    check_with_warning(ddf3, df1, df3, df1)
+    check_append_with_warning(ddf1, df3, df1, df3)
+    check_append_with_warning(ddf3, df1, df3, df1)
 
 
 def test_append_categorical():
@@ -2133,16 +2213,16 @@ def test_append_categorical():
             )
         ddf1, ddf2 = dframes
 
-        res = check_with_warning(ddf1, ddf2, df1, df2)
+        res = check_append_with_warning(ddf1, ddf2, df1, df2)
 
         assert has_known_categories(res.index) == known
         assert has_known_categories(res.y) == known
 
-        res = check_with_warning(ddf1.y, ddf2.y, df1.y, df2.y)
+        res = check_append_with_warning(ddf1.y, ddf2.y, df1.y, df2.y)
         assert has_known_categories(res.index) == known
         assert has_known_categories(res) == known
 
-        res = check_with_warning(ddf1.index, ddf2.index, df1.index, df2.index)
+        res = check_append_with_warning(ddf1.index, ddf2.index, df1.index, df2.index)
         assert has_known_categories(res) == known
 
 
@@ -2150,7 +2230,7 @@ def test_append_lose_divisions():
     df = pd.DataFrame({"x": [1, 2, 3, 4]}, index=[1, 2, 3, 4])
     ddf = dd.from_pandas(df, npartitions=2)
 
-    res = check_with_warning(ddf, ddf, df, df)
+    res = check_append_with_warning(ddf, ddf, df, df)
     assert res.known_divisions is False
 
 
@@ -2178,7 +2258,7 @@ def test_multi_duplicate_divisions():
 
     ddf1 = dd.from_pandas(df1, npartitions=2).set_index("x")
     ddf2 = dd.from_pandas(df2, npartitions=1).set_index("x")
-    assert ddf1.npartitions == 2
+    assert ddf1.npartitions <= 2
     assert len(ddf1) == len(df1)
 
     r1 = ddf1.merge(ddf2, how="left", left_index=True, right_index=True)
@@ -2213,10 +2293,9 @@ def test_dtype_equality_warning():
     df1 = pd.DataFrame({"a": np.array([1, 2], dtype=np.dtype(np.int64))})
     df2 = pd.DataFrame({"a": np.array([1, 2], dtype=np.dtype(np.longlong))})
 
-    with pytest.warns(None) as r:
+    with warnings.catch_warnings(record=True) as record:
         dd.multi.warn_dtype_mismatch(df1, df2, "a", "a")
-
-    assert len(r) == 0
+    assert not record
 
 
 @pytest.mark.parametrize(
