@@ -344,7 +344,7 @@ def concrete(seq):
     return seq
 
 
-def pseudorandom(n, p, random_state=None):
+def pseudorandom(n: int, p, random_state=None):
     """Pseudorandom array of integer indexes
 
     >>> pseudorandom(5, [0.5, 0.5], random_state=123)
@@ -371,7 +371,7 @@ def pseudorandom(n, p, random_state=None):
     return out
 
 
-def random_state_data(n, random_state=None):
+def random_state_data(n: int, random_state=None) -> list:
     """Return a list of arrays that can initialize
     ``np.random.RandomState``.
 
@@ -395,7 +395,7 @@ def random_state_data(n, random_state=None):
     return l
 
 
-def is_integer(i):
+def is_integer(i) -> bool:
     """
     >>> is_integer(6)
     True
@@ -529,7 +529,7 @@ def takes_multiple_arguments(func, varargs=True):
     return len(spec.args) - ndefaults - is_constructor > 1
 
 
-def get_named_args(func):
+def get_named_args(func) -> list[str]:
     """Get all non ``*args/**kwargs`` arguments for a function"""
     s = inspect.signature(func)
     return [
@@ -613,7 +613,7 @@ class Dispatch:
             return "Single Dispatch for %s" % self.__name__
 
 
-def ensure_not_exists(filename):
+def ensure_not_exists(filename) -> None:
     """
     Ensure that a file does not exist.
     """
@@ -820,7 +820,7 @@ def derived_from(original_klass, version=None, ua_args=None, skipblocks=0):
     return wrapper
 
 
-def funcname(func):
+def funcname(func) -> str:
     """Get the name of a function."""
     # functools.partial
     if isinstance(func, functools.partial):
@@ -852,7 +852,7 @@ def funcname(func):
         return str(func)[:50]
 
 
-def typename(typ, short=False):
+def typename(typ, short=False) -> str:
     """
     Return the name of a type
 
@@ -882,7 +882,7 @@ def typename(typ, short=False):
         return str(typ)
 
 
-def ensure_bytes(s):
+def ensure_bytes(s) -> bytes:
     """Turn string or bytes to bytes
 
     >>> ensure_bytes('123')
@@ -900,8 +900,8 @@ def ensure_bytes(s):
     raise TypeError(msg % s)
 
 
-def ensure_unicode(s):
-    """Turn string or bytes to bytes
+def ensure_unicode(s) -> str:
+    """Turn string or bytes to string
 
     >>> ensure_unicode('123')
     '123'
@@ -1228,7 +1228,7 @@ def partial_by_order(*args, **kwargs):
     return function(*args2, **kwargs)
 
 
-def is_arraylike(x):
+def is_arraylike(x) -> bool:
     """Is this object a numpy array or something similar?
 
     This function tests specifically for an object that already has
@@ -1271,7 +1271,7 @@ def is_arraylike(x):
     )
 
 
-def is_dataframe_like(df):
+def is_dataframe_like(df) -> bool:
     """Looks like a Pandas DataFrame"""
     if (df.__class__.__module__, df.__class__.__name__) == (
         "pandas.core.frame",
@@ -1287,7 +1287,7 @@ def is_dataframe_like(df):
     )
 
 
-def is_series_like(s):
+def is_series_like(s) -> bool:
     """Looks like a Pandas Series"""
     typ = s.__class__
     return (
@@ -1297,7 +1297,7 @@ def is_series_like(s):
     )
 
 
-def is_index_like(s):
+def is_index_like(s) -> bool:
     """Looks like a Pandas Index"""
     typ = s.__class__
     return (
@@ -1306,7 +1306,7 @@ def is_index_like(s):
     )
 
 
-def is_cupy_type(x):
+def is_cupy_type(x) -> bool:
     # TODO: avoid explicit reference to CuPy
     return "cupy" in str(type(x))
 
@@ -1349,7 +1349,7 @@ def factors(n):
     return set(functools.reduce(list.__add__, seq))
 
 
-def parse_bytes(s):
+def parse_bytes(s: float | str) -> int:
     """Parse byte string to numbers
 
     >>> from dask.utils import parse_bytes
@@ -1570,7 +1570,7 @@ timedelta_sizes.update(tds2)
 timedelta_sizes.update({k.upper(): v for k, v in timedelta_sizes.items()})
 
 
-def parse_timedelta(s, default="seconds"):
+def parse_timedelta(s: float | str | timedelta | None, default: str = "seconds"):
     """Parse timedelta string to number of seconds
 
     Examples
